# Note.py

# ========================================
# note classes
# ========================================


class Note:
    """Note class corresponds to Csound orchestra instrument template"""
    
    def __init__(self, startTime, duration, pitch, pitch2, amp, envType, *args):
        """Accepts arbitrary extra values (pFields) starting from p9"""
        self.startTime = startTime
        self.duration = duration
        self.pitch = pitch
        self.pitch2 = pitch2
        self.amp = amp
        self.envType = envType
        self.extraPFields = list(args)
        
        self.modifier = None
               
    def setModifier(self, modifier):
        self.modifier = modifier
        
    def getEndTime(self):
        return self.startTime + self.duration
    
    def getNote(self, instrumentNum, space):
        """Generates a Csound noteString.  startTime and space should be passed
        in from Performer"""
        
        if self.modifier != None:
            return self.modifier.generateScore(self, instrumentNum, space)

        tempPFields = [instrumentNum, self.startTime, self.duration] 
        tempPFields += [self.pitch, self.pitch2, self.amp, space, self.envType]
        tempPFields += self.extraPFields
        
        tempPFields = [str(i).ljust(15) for i in tempPFields]

        #return "i " + " ".join(map(str,tempPFields))
        return "i " + " ".join(tempPFields)


class Rest:
    """Rest class produces no noteText but is useful for putting in space in
    music lines"""
    
    def __init__(self, startTime, duration):
        self.startTime = startTime
        self.duration = duration
        self.hasFermata = 0
        self.pitch = 0
        self.pitch2 = 0
        self.amp = 0
        self.envType = 0
        self.extraPFields = []
        
    def setModifier(self, modifier):
        pass

    def getEndTime(self):
        return self.startTime + self.duration
        
    def getNote(self, instrumentNum, space):
        return ""
