import string
import copy

"""Utility functions for score related operations"""

def getBaseTen(pch):
    """Given a pch notation string, return value in base 10"""
    pchParts = string.split(str(pch), ".")
    octave  = int(pchParts[0])

    if len(pchParts) == 1:
        return octave * 12

    pitch   = float("." + pchParts[1])
    pitch   = 100 * pitch

    baseTen = (octave * 12) + pitch

    return baseTen
    
def getPch(baseTen):
    octave  = int(baseTen / 12)
    pitch   = float(baseTen % 12) / 100

    return octave + pitch
        


def pchAdd(pch, addValue):
    """given a pch, add interval to that pch
    
    example:
    
    pchAdd(7.11, 2)
    
    will return 8.01
    
    """

    baseTen = getBaseTen(pch)
    baseTen += float(addValue)

    return getPch(baseTen)

def pchDiff(pch1, pch2):
    """Returns the number of steps between two pch's"""

    baseTen1    = getBaseTen(pch1)
    baseTen2    = getBaseTen(pch2)

    diff    = baseTen2 - baseTen1
    
    return diff
  
def makePchSet(startPch, intervalList):
    """Given a starting pch and a list of intervals, 
    returns a list of pch's"""
    
    retVal = [startPch]
    
    currentPch = startPch
    
    for interval in intervalList:
        currentPch = pchAdd(currentPch, interval)
        retVal.append(currentPch)
        
    return retVal
  
    
def makeMirror(intervalList):
    """Given an intervalList, creates a mirror of that list"""
    mirror = copy.deepcopy(intervalList)
    for i in range(len(intervalList)):
        mirror.append(-intervalList[-i -1])
        
    return mirror


